#include <cstdio>
#include <queue>
#include <vector>
#include <algorithm>
#include <utility>

using namespace std;

const int MAXN = 500;

struct Edge {
  int u, v, d;
  Edge(int uu = 0, int vv = 0, int dd = 0) : u(uu), v(vv), d(dd) {}
  bool operator<(const Edge& rhs) const { return d < rhs.d; }
  bool operator>(const Edge& rhs) const { return d > rhs.d; }
};

int dist[MAXN][MAXN], n, uf[MAXN];
vector<Edge> g[MAXN];
vector<int> tree[MAXN], clients[MAXN];
bool seen[MAXN];

void search(int v) {
  priority_queue<Edge, vector<Edge>, greater<Edge> > q;

  q.push(Edge(-1,v,0));
  while (!q.empty()) {
    Edge curr = q.top();
    q.pop();
    if (dist[v][curr.v] != -1) continue;
    dist[v][curr.v] = curr.d;
    for (int i = 0; i < g[curr.v].size(); ++i) {
      Edge nxt(curr.v, g[curr.v][i].v, g[curr.v][i].d+curr.d);
      if (dist[v][nxt.v] == -1) q.push(nxt);
    }
  }
}

int parent(int v) {
  if (uf[v] == v) return v;
  else return uf[v] = parent(uf[v]);
}

bool merge(int u, int v) {
  u = parent(u);
  v = parent(v);
  if (u == v) return false;
  uf[u] = v;
  return true;
}

void dfs(int v, bool& first) {
  if (seen[v]) return;
  seen[v] = true;
  for (int i = 0; i < clients[v].size(); ++i) {
    if (first) first = false;
    else printf(" ");
    printf("%d", clients[v][i]);
  }
  for (int i = 0; i < tree[v].size(); ++i) dfs(tree[v][i], first);
}

void travel(const vector<int>& b, bool& first) {
  if (b.size() == 0) return;
  vector<Edge> e;
  int mn = b[0];
  for (int i = 0; i < b.size(); ++i) {
    if (dist[0][mn] > dist[0][b[i]]) mn = b[i];
    for (int j = 0; j < i; ++j)
      e.push_back(Edge(b[i], b[j], dist[b[i]][b[j]]));
  }

  sort(e.begin(), e.end());
  
  for (int i = 0; i < e.size(); ++i)
    if (merge(e[i].u, e[i].v)) {
      tree[e[i].u].push_back(e[i].v);
      tree[e[i].v].push_back(e[i].u);
    }

  dfs(mn, first);
}

int main() {
  int t;
  scanf("%d", &t);
  while (t--) {
    int m, k;
    scanf("%d%d%d", &n, &m, &k);
    for (int i = 0; i < n; ++i) {
      g[i].clear();
      clients[i].clear();
      for (int j = 0; j < n; ++j)
	dist[i][j] = -1;
    }

    for (int i = 0; i < k; ++i) {
      int v;
      scanf("%d", &v);
      clients[v].push_back(i+1);
    }

    while (m--) {
      int u,v,d;
      scanf("%d%d%d", &u, &v, &d);
      g[u].push_back(Edge(u,v,d));
      g[v].push_back(Edge(v,u,d));
    }

    for (int i = 0; i < n; ++i) search(i);
    
    vector< pair<double, int> > weight;

    for (int i = 0; i < n; ++i) {
      uf[i] = i;
      seen[i] = false;
      tree[i].clear();
      if (clients[i].size() != 0)
	weight.push_back(pair<double, int>(dist[i][0]/double(clients[i].size()), i));
    }
    sort(weight.begin(), weight.end());
    vector< vector<int> > bucket(1);
    double mn = weight[0].first;
    bucket[0].push_back(weight[0].second);
    
    for (int i = 1; i < weight.size(); ++i) {
      if (weight[i].first > 1.1*mn) {
	bucket.push_back(vector<int>());
	mn = weight[i].first;
      }
      bucket[bucket.size()-1].push_back(weight[i].second);
    }

    bool first = true;
    for (int i = 0; i < bucket.size(); ++i)
      travel(bucket[i], first);
    printf("\n");
  }
}